@@ -38,7 +38,10 @@
 #define  POLAR_DX         230.0     ///< [mas] Earth polar offset x, e.g. from IERS Bulletin A.
 #define  POLAR_DY         -62.0     ///< [mas] Earth polar offset y, e.g. from IERS Bulletin A.
 
-int main() {
+int main(int argc, char *argv[]) {
+  // Program Options -------------------------------------------------------->
+  char *datafile = "/path/to/de440s.bsp";  // Ephemeris file to use
+
   // SuperNOVAS variables used for the calculations ------------------------->
   object source;                    // observed source
   observer obs;                     // observer location
@@ -52,9 +55,12 @@ int main() {
 
   // Intermediate variables we'll use -------------------------------------->
   t_calcephbin *de440;              // CALCEPH ephemeris binary
-  struct timespec unix_time;        // Standard precision UNIX time structure
 
 
+  // Command line argument can define the ephemeris data to use.
+  if(argc > 1)
+    datafile = argv[1];
+
   // We'll print debugging messages and error traces...
   novas_debug(NOVAS_DEBUG_ON);
 
@@ -63,7 +69,7 @@ int main() {
 
   // First open one or more ephemeris files with CALCEPH to use
   // E.g. the DE440 (short-term) ephemeris data from JPL.
-  de440 = calceph_open("path/to/de440s.bsp");
+  de440 = calceph_open(datafile);
   if(!de440) {
     fprintf(stderr, "ERROR! could not open ephemeris data\n");
     return 1;
@@ -114,9 +120,9 @@ int main() {
   // or Sun orbit, at the geocenter or at the Solary-system barycenter...)
 
   // Specify the location we are observing from
-  // 50.7374 deg N, 7.0982 deg E, 60m elevation
-  // (We'll ignore the local weather parameters here, but you can set those too.)
-  if(make_observer_on_surface(50.7374, 7.0982, 60.0, 0.0, 0.0, &obs) != 0) {
+  // 50.7374 deg N, 7.0982 deg E, 60m elevation (GPS / WGS84)
+  // (You can set local weather parameters after...)
+  if(make_gps_observer(50.7374, 7.0982, 60.0, &obs) != 0) {
     fprintf(stderr, "ERROR! defining Earth-based observer location.\n");
     return 1;
   }
@@ -125,11 +131,8 @@ int main() {
   // -------------------------------------------------------------------------
   // Set the astrometric time of observation...
 
-  // Get the current system time, with up to nanosecond resolution...
-  clock_gettime(CLOCK_REALTIME, &unix_time);
-
-  // Set the time of observation to the precise UTC-based UNIX time
-  if(novas_set_unix_time(unix_time.tv_sec, unix_time.tv_nsec, LEAP_SECONDS, DUT1, &obs_time) != 0) {
+  // Set the time of observation to the current UTC-based UNIX time
+  if(novas_set_current_time(LEAP_SECONDS, DUT1, &obs_time) != 0) {
     fprintf(stderr, "ERROR! failed to set time of observation.\n");
     return 1;
   }
