@@ -53,10 +53,6 @@ int main() {
   // Calculated quantities ------------------------------------------------->
   double az, el;                    // calculated azimuth and elevation at observing site
 
-  // Intermediate variables we'll use -------------------------------------->
-  struct timespec unix_time;        // Standard precision UNIX time structure
-
-
   // We'll print debugging messages and error traces...
   novas_debug(NOVAS_DEBUG_ON);
 
@@ -116,9 +112,9 @@ int main() {
   // or Sun orbit, at the geocenter or at the Solary-system barycenter...)
 
   // Specify the location we are observing from
-  // 50.7374 deg N, 7.0982 deg E, 60m elevation
-  // (We'll ignore the local weather parameters here, but you can set those too.)
-  if(make_observer_on_surface(50.7374, 7.0982, 60.0, 0.0, 0.0, &obs) != 0) {
+  // 50.7374 deg N, 7.0982 deg E, 60m elevation (GPS / WGS84)
+  // (You can set local weather parameters after...)
+  if(make_gps_observer(50.7374, 7.0982, 60.0, &obs) != 0) {
     fprintf(stderr, "ERROR! defining Earth-based observer location.\n");
     return 1;
   }
@@ -127,12 +123,9 @@ int main() {
   // -------------------------------------------------------------------------
   // Set the astrometric time of observation...
 
-  // Get the current system time, with up to nanosecond resolution...
-  clock_gettime(CLOCK_REALTIME, &unix_time);
-
-  // Set the time of observation to the precise UTC-based UNIX time
+  // Set the time of observation to the current UTC-based UNIX time
   // (We can set astromtric time using an other time measure also...)
-  if(novas_set_unix_time(unix_time.tv_sec, unix_time.tv_nsec, LEAP_SECONDS, DUT1, &obs_time) != 0) {
+  if(novas_set_current_time(LEAP_SECONDS, DUT1, &obs_time) != 0) {
     fprintf(stderr, "ERROR! failed to set time of observation.\n");
     return 1;
   }
