@@ -185,18 +185,16 @@ short cio_location(double jd_tdb, enum novas_accuracy accuracy, double *restrict
     return novas_error(-1, EINVAL, fn, "invalid accuracy: %d", accuracy);
 
   // Check if previously computed RA value can be used.
-  if(novas_time_equals(jd_tdb, t_last) && (accuracy == acc_last)) {
-    *ra_cio = ra_last;
-    return 0;
+  if(!novas_time_equals(jd_tdb, t_last) || accuracy != acc_last) {
+    // Calculate the equation of origins.
+    *ra_cio = -ira_equinox(jd_tdb, NOVAS_TRUE_EQUINOX, accuracy);
+
+    t_last = jd_tdb;
+    acc_last = accuracy;
+    ra_last = *ra_cio;
   }
 
-  // Calculate the equation of origins.
-  *ra_cio = -ira_equinox(jd_tdb, NOVAS_TRUE_EQUINOX, accuracy);
-
-  t_last = jd_tdb;
-  acc_last = accuracy;
-  ra_last = *ra_cio;
-
+  *ra_cio = ra_last;
   return 0;
 }
 
@@ -254,7 +252,7 @@ short cio_basis(double jd_tdb, double ra_cio, enum novas_cio_location_type loc_t
     return novas_error(-1, EINVAL, fn, "invalid accuracy: %d", accuracy);
 
   // Compute unit vector z toward celestial pole.
-  if(!zz[2] || !novas_time_equals(jd_tdb, t_last) || (accuracy != acc_last)) {
+  if(!novas_time_equals(jd_tdb, t_last) || (accuracy != acc_last)) {
     const double z0[3] = { 0.0, 0.0, 1.0 };
     tod_to_gcrs(jd_tdb, accuracy, z0, zz);
     t_last = jd_tdb;
