@@ -1,42 +1,117 @@
 /**
  * @file
  *
- * @author A. Kovacs
- * @since 1.2
+ *  Solar-system ephemeris lookup via the CALCEPH C library.
  *
- *  SuperNOVAS Solar-system ephemeris lookup implementation via the CALCEPH C library.
+ *  This is an optional component of SuperNOVAS, which interfaces to the CALCEPH C library. As
+ *  such, you may need the CALCEPH runtime libraries in an accessible location (such as in
+ *  `/usr/lib`) to use, and you will need development files (C headers and unversioned libraries)
+ *  to build. Thus, this module is compiled only if `CALCEPH_SUPPORT` is set to 1 prior to the
+ *  build.
  *
- *  This is an optional component of SuperNOVAS, which interfaces to the CALCEPH C library.
- *  As such, you may need the CALCEPH runtime libraries in an accessible location (such
- *  as in `/usr/lib`) to use, and you will need development files (C headers and unversioned
- *  libraries) to build. Thus, this module is compiled only if `CALCEPH_SUPPORT` is set
- *  to 1 prior to the build.
+ *  To use, simply include `novas-calceph.h` in your application source, configure a
+ *  `t_calcephbin` object with the requisite ephemeris data, and then call `novas_use_calceph()`
+ *  with it to activate. E.g.:
+ *
+ *  ```c
+ *    #include <novas.h>
+ *    #include <novas-calceph.h>
+ *
+ *    // You can open a set of JPL/INPOP ephemeris files with CALCEPH...
+ *    t_calcephbin *eph = calceph_open_array(...);
+ *
+ *    // Then use them as your generic SuperNOVAS ephemeris provider
+ *    int status = novas_use_calceph(eph);
+ *    if(status < 0) {
+ *      // Oops something went wrong...
+ *    }
+ *  ```
+ *
+ * Optionally, you may use a separate ephemeris dataset for major planets (or if planet
+ * ephemeris was included in 'eph' above, you don't have to):
+ *
+ *  ```c
+ *    t_calcephbin *pleph = calceph_open(...);
+ *    status = novas_use_calceph_planets(pleph);
+ *    if(status < 0) {
+ *      // Oops something went wrong...
+ *    }
+ *  ```
+ *
+ * By default the CALCEPH plugin will use NAIF ID numbers for the lookup (for planets the
+ * NOVAS IDs will be mapped to NAIF IDs automatically). You can enable name-based lookup by
+ * setting the @ref object number to -1 (e.g. in `make_ephem_object()`), or else switch to
+ * using CALCEPH IDs by calling `novas_calceph_use_ids(NOVAS_ID_CALCEPH)`.
  *
  * REFERENCES:
  * <ol>
- *  <li>CALCEPH is at https://calceph.imcce.fr/docs/4.0.0/html/c/</li>
+ *  <li>CALCEPH is at https://calceph.imcce.fr</li>
  *  <li>CALCEPH source code is at https://gitlab.obspm.fr/imcce_calceph/calceph</li>
  * </ol>
  *
- * @sa solarsystem.h
- * @sa solsys-cspice.c
+ * @author Attila Kovacs
+ * @since 1.2
+ *
+ * @sa solsys-cspice.c, ephemeris.c, orbital.c
  */
 
 #include <string.h>
 #include <errno.h>
-#include <semaphore.h>
 
 /// \cond PRIVATE
+#if defined(SUPERNOVAS_USE_PTHREAD) || defined(__unix__) || defined(__unix) || defined(__APPLE__)
+#  include <pthread.h>
+
+#  define init_lock           pthread_mutex_init
+#  define ephem_lock          pthread_mutex_lock
+#  define ephem_unlock        pthread_mutex_unlock
+#  define THREAD_SAFE         1
+
+typedef pthread_mutex_t       lock_type;
+
+#elif __STDC_VERSION__ >= 201112L
+#  include <threads.h>
+
+#  define init_lock           mtx_init
+#  define ephem_lock          mtx_lock
+#  define ephem_unlock        mtx_unlock
+
+#  define THREAD_SAFE         1
+
+typedef mtx_t                 lock_type;
+
+#elif defined(WIN32)
+#include <windows.h>
+
+#  define init_lock(x, t)     InitializeSRWLock(x)
+#  define ephem_lock          AcquireSRWLockExclusive
+#  define ephem_unlock        ReleaseSRWLockExclusive
+#  define THREAD_SAFE         1
+
+typedef SRWLOCK               lock_type;
+
+#else
+#  define ephem_lock(x)
+#  define ephem_unlock(x)
+#  define THREAD_SAFE         0
+typedef int                   lock_type;
+
+#endif
+
 #define __NOVAS_INTERNAL_API__      ///< Use definitions meant for internal use by SuperNOVAS only
+/// \endcond
+
 #include "novas.h"
 #include "novas-calceph.h"
 
+
+/// \cond PRIVATE
 #define CALCEPH_MOON                10  ///< Moon in CALCEPH
 #define CALCEPH_SUN                 11  ///< Sun in CALCEPH
 #define CALCEPH_SSB                 12  ///< Solar-system Barycenter in CALCEPH
 
-/// Distance and time units to use for CALCEPH (AU would be conventient, but is not available
-/// unless defined in the sphemeris file(s) themselves.
+/// Distance and time units to use for CALCEPH (AU would be convenient, but is not available
+/// unless defined in the ephemeris file(s) themselves.
 #define CALCEPH_UNITS               (CALCEPH_UNIT_KM | CALCEPH_UNIT_DAY)
 
 /// Multiplicative normalization for the positions returned by CALCEPH to AU
@@ -44,43 +119,24 @@
 
 /// Multiplicative normalization for the velocities returned by CALCEPH to AU/day
 #define NORM_VEL                    (NORM_POS)
+/// \endcond
 
 /// Whether to force serialized (non-parallel CALCEPH queries)
-int serialized_calceph_queries;
-
-/// \endcond
+int serialized_calceph_queries = 1;
 
 static int compute_flags = CALCEPH_USE_NAIFID;
 
 /// CALCEPH ephemeris specifically for planets (and Sun and Moon) only
 static t_calcephbin *planets;
 
-/// (boolean) whether the planets ephemeris data is thread safe to access
-static int is_thread_safe_planets;
-
-/// Semaphore for thread-safe access of planet ephemeris (if needed)
-static sem_t sem_planets;
-
 /// Generic CALCEPH ephemeris files for all types of Solar-system sources
 static t_calcephbin *bodies;
 
-/// (boolean) whether the generic solar-system bodies ephemeris data is thread safe to access
-static int is_thread_safe_bodies;
+/// Semaphore for thread-safe access of planet ephemeris (if needed)
+static lock_type planet_mutex;
 
 /// Semaphore for thread-safe access of generic solar-system bodies ephemeris (if needed)
-static sem_t sem_bodies;
-
-
-static int mutex_lock(sem_t *sem) {
-  if(sem_wait(sem) != 0)
-    return novas_error(-1, errno, "mutex_lock()", "sem_wait()");
-  return 0;
-}
-
-static int mutex_unlock(sem_t *sem) {
-  sem_post(sem);
-  return 0;
-}
+static lock_type bodies_mutex;
 
 static int prep_ephem(t_calcephbin *eph) {
   static const char *fn = "prep_ephem";
@@ -94,6 +150,17 @@ static int prep_ephem(t_calcephbin *eph) {
   return 0;
 }
 
+/**
+ * Checks if the CALCEPH plugin is thread safe.
+ *
+ * @return      TRUE (1) if the lugin is thread safe, or else FALSE (0).
+ *
+ * @since 1.5
+ */
+int novas_calceph_is_thread_safe() {
+  return THREAD_SAFE;
+}
+
 /**
  * Sets the type of Solar-system body IDs to use as object.number with NOVAS_EPHEM_OBJECT types.
  * CALCEPH supports the use of both NAIF and its own numbering system to identify Solar-system
@@ -103,10 +170,10 @@ static int prep_ephem(t_calcephbin *eph) {
  *
  * @param idtype    NOVAS_ID_NAIF to use NAIF IDs (default) or else NOVAS_ID_CALCEPH to use
  *                  the CALCEPH body numbering convention for object.
- * @return          0 if successful or else -1 (errno set to EINVAL) if the input value is invalid.
+ * @return          0 if successful or else -1 (errno set to EINVAL) if the input value is
+ *                  invalid.
  *
- * @sa object
- * @sa NOVAS_EPHEM_OBJECT
+ * @sa make_planet(), make_ephem_object(), NOVAS_EPHEM_OBJECT
  */
 int novas_calceph_use_ids(enum novas_id_type idtype) {
   switch(idtype) {
@@ -121,7 +188,6 @@ int novas_calceph_use_ids(enum novas_id_type idtype) {
   }
 }
 
-
 /**
  * Provides an interface between the CALCEPH C library and NOVAS-C for regular (reduced) precision
  * applications. The user must set the CALCEPH ephemeris binary data to use using the
@@ -133,16 +199,16 @@ int novas_calceph_use_ids(enum novas_id_type idtype) {
  *
  * REFERENCES:
  * <ol>
- *  <li>The CALCEPH C library; https://calceph.imcce.fr/docs/4.0.0/html/c/index.html</li>
+ *  <li>The CALCEPH C library; https://calceph.imcce.fr</li>
  *  <li>Kaplan, G. H. "NOVAS: Naval Observatory Vector Astrometry
  *  Subroutines"; USNO internal document dated 20 Oct 1988;
  *  revised 15 Mar 1990.</li>
  * </ol>
  *
- * @param jd_tdb         [day] Two-element array containing the Julian date, which may be
- *                       split any way (although the first element is usually the
- *                       "integer" part, and the second element is the "fractional"
- *                       part).  Julian date is on the TDB or "T_eph" time scale.
+ * @param jd_tdb         [day] Two-element array containing the Julian date, which may be split
+ *                       any way (although the first element is usually the "integer" part, and
+ *                       the second element is the "fractional" part).  Julian date is on the TDB
+ *                       or "T_eph" time scale.
  * @param body           Major planet number (or that for Sun, Moon, SSB...)
  * @param origin         NOVAS_BARYCENTER (0) or NOVAS_HELIOCENTER (1)
  *                       -- relative to which to report positions and velocities.
@@ -154,19 +220,17 @@ int novas_calceph_use_ids(enum novas_id_type idtype) {
  *                       'origin' is invalid, or 3 if there was an error providing ephemeris
  *                       data.
  *
- * @sa planet_calceph
- * @sa novas_use_calceph()
- * @sa novas_use_calceph_planet()
- *
  * @author Attila Kovacs
  * @since 1.2
+ *
+ * @sa planet_calceph(), novas_use_calceph(), novas_use_calceph_planet()
  */
 static short planet_calceph_hp(const double jd_tdb[restrict 2], enum novas_planet body, enum novas_origin origin,
         double *restrict position, double *restrict velocity) {
   static const char *fn = "planet_calceph_hp";
 
-  sem_t *sem = (planets == bodies) ? &sem_bodies : &sem_planets;
-  const int lock = !is_thread_safe_planets || serialized_calceph_queries;
+  t_calcephbin *ephem;
+  int parallel;
   double pv[6] = {0.0};
   int i, target, center, success;
 
@@ -200,13 +264,23 @@ static short planet_calceph_hp(const double jd_tdb[restrict 2], enum novas_plane
       return novas_error(2, EINVAL, fn, "Invalid origin type: %d", origin);
   }
 
-  if(lock)
-    prop_error(fn, mutex_lock(sem), 0);
+  ephem_lock(&planet_mutex);
+  ephem = planets;
 
-  success = calceph_compute_unit(planets, jd_tdb[0], jd_tdb[1], target, center, CALCEPH_UNITS, pv);
+  parallel = !serialized_calceph_queries && calceph_isthreadsafe(ephem);
+  if(parallel)
+    ephem_unlock(&planet_mutex);      // If CALCEPH itself is thread-safe we can release the lock here...
+  else if(ephem == bodies)
+    ephem_lock(&bodies_mutex);        // Otherwise, we might need to get exclusive on the bodies also....
 
-  if(lock)
-   mutex_unlock(sem);
+  success = calceph_compute_unit(ephem, jd_tdb[0], jd_tdb[1], target, center, CALCEPH_UNITS, pv);
+
+  // If CALCEPH is not thread-safe on its own, release the lock after the ephemeris access...
+  if(!parallel) {
+    if(ephem == bodies)
+      ephem_unlock(&bodies_mutex);    // If no separate planet ephemeris, then release the lock on the bodies also.
+    ephem_unlock(&planet_mutex);      // release the lock on planet ephemeris access.
+  }
 
   if(!success)
     return novas_error(3, EAGAIN, fn, "calceph_compute() failure (NOVAS ID=%d)", body);
@@ -223,25 +297,26 @@ static short planet_calceph_hp(const double jd_tdb[restrict 2], enum novas_plane
 
 /**
  * Provides an interface between the CALCEPH C library and NOVAS-C for regular (reduced) precision
- * applications. The user must set the CALCEPH ephemeris binary data to use using the
- * novas_use_calceph() or novas_use_calceph_planet() to activate the desired CALCEPH ephemeris
- * data prior to use.
+ * applications, but in reality it's exactly the same as the high-precision version, except for
+ * the way the TDB-based Julian date is specified. The user must set the CALCEPH ephemeris binary
+ * data to use using the novas_use_calceph() or novas_use_calceph_planet() to activate the desired
+ * CALCEPH ephemeris data prior to use.
  *
  * This call is always thread safe, even when CALCEPH and the ephemeris data may not be. When
  * necessary, the ephemeris access will be mutexed to ensure sequential access under the hood.
  *
  * REFERENCES:
  * <ol>
- *  <li>The CALCEPH C library; https://calceph.imcce.fr/docs/4.0.0/html/c/index.html</li>
+ *  <li>The CALCEPH C library; https://calceph.imcce.fr</li>
  *  <li>Kaplan, G. H. "NOVAS: Naval Observatory Vector Astrometry
  *  Subroutines"; USNO internal document dated 20 Oct 1988;
  *  revised 15 Mar 1990.</li>
  * </ol>
  *
- * @param jd_tdb         [day] Two-element array containing the Julian date, which may be
- *                       split any way (although the first element is usually the
- *                       "integer" part, and the second element is the "fractional"
- *                       part).  Julian date is on the TDB or "T_eph" time scale.
+ * @param jd_tdb         [day] Two-element array containing the Julian date, which may be split
+ *                       any way (although the first element is usually the "integer" part, and
+ *                       the second element is the "fractional" part).  Julian date is on the TDB
+ *                       or "T_eph" time scale.
  * @param body           Major planet number (or that for Sun, Moon, SSB...)
  * @param origin         NOVAS_BARYCENTER (0) or NOVAS_HELIOCENTER (1), or 2 for Earth geocenter
  *                       -- relative to which to report positions and velocities.
@@ -249,16 +324,12 @@ static short planet_calceph_hp(const double jd_tdb[restrict 2], enum novas_plane
  *                       coordinates in AU referred to the ICRS.
  * @param[out] velocity  [AU/day] Velocity vector of 'body' at jd_tdb; equatorial rectangular
  *                       system referred to the ICRS, in AU/day.
- * @return               0 if successful, or else an error code of solarsystem().
- *
- * @sa planet_eph_manager_hp()
- * @sa planet_ephem_provider()
- * @sa ephem_open()
- * @sa set_planet_provider()
- * @sa solarsystem()
+ * @return               0 if successful, or else an error code defined by novas_planet_provider.
  *
  * @author Attila Kovacs
  * @since 1.2
+ *
+ * @sa set_planet_provider(), planet_calceph_hp()
  */
 static short planet_calceph(double jd_tdb, enum novas_planet body, enum novas_origin origin,
         double *restrict position, double *restrict velocity) {
@@ -279,7 +350,7 @@ static short planet_calceph(double jd_tdb, enum novas_planet body, enum novas_or
  *                      position in desired, or -1 if the 'name' should be used instead to
  *                      identify the object.
  * @param jd_tdb_high   [day] The high-order part of Barycentric Dynamical Time (TDB) based
- *                      Julian date for which to find the position and velocity. Typically
+ *                      Julian date for which to find the position and velocityhttps://localhost:8443/. Typically
  *                      this may be the integer part of the Julian date for high-precision
  *                      calculations, or else the entire Julian date for reduced precision.
  * @param jd_tdb_low    [day] The low-order part of Barycentric Dynamical Time (TDB) based
@@ -295,26 +366,24 @@ static short planet_calceph(double jd_tdb, enum novas_planet body, enum novas_or
  *                      coordinates in AU/day. It may be NULL if velocities are not required.
  * @return              0 if successful, -1 if any of the pointer arguments are NULL, or some
  *                      non-zero value if the was an error s.t. the position and velocity
- *                      vector should not be used.
+ *                      vector should not be used (errno set to EINVAL); or if no
+ *                      ephemerides have been configured (errno set to EAGAIN).
  *
- * @sa novas_calceph_use_ids()
- * @sa set_ephem_provider()
- * @sa ephemeris()
- * @sa NOVAS_EPHEM_OBJECT
- *
- *@author Attila Kovacs
+ * @author Attila Kovacs
  * @since 1.2
+ *
+ * @sa novas_calceph_use_ids(), make_ephem_object(), set_ephem_provider(), ephemeris()
  */
 static int novas_calceph(const char *name, long id, double jd_tdb_high, double jd_tdb_low, enum novas_origin *origin, double *pos, double *vel) {
   static const char *fn = "novas_calceph";
 
+  t_calcephbin *ephem;
+  int parallel;
   double pv[6] = {0.0};
-  const int lock = !is_thread_safe_bodies || serialized_calceph_queries;
   int i, success, center;
 
   if(id == -1) {
     // Lookup by name...
-
     if(!name)
       return novas_error(-1, EINVAL, fn, "id=-1 and name is NULL");
 
@@ -334,13 +403,19 @@ static int novas_calceph(const char *name, long id, double jd_tdb_high, double j
 
   center = (compute_flags & CALCEPH_USE_NAIFID) ? NAIF_SSB : CALCEPH_SSB;
 
-  if(lock)
-    prop_error(fn, mutex_lock(&sem_bodies), 0);
+  ephem_lock(&bodies_mutex);
+  ephem = bodies;
 
-  success = calceph_compute_unit(bodies, jd_tdb_high, jd_tdb_low, id, center, (compute_flags | CALCEPH_UNITS), pv);
+  // If CALCEPH itself is thread-safe we can release the lock here...
+  parallel = !serialized_calceph_queries && calceph_isthreadsafe(ephem);
+  if(parallel)
+    ephem_unlock(&bodies_mutex);
 
-  if(lock)
-    mutex_unlock(&sem_bodies);
+  success = calceph_compute_unit(ephem, jd_tdb_high, jd_tdb_low, id, center, (compute_flags | CALCEPH_UNITS), pv);
+
+  // If CALCEPH is not thread-safe on its own, release the lock after the ephemeris access...
+  if(!parallel)
+    ephem_unlock(&bodies_mutex);
 
   if(!success)
     return novas_error(3, EAGAIN, fn, "calceph_compute() failure (name='%s', NAIF=%ld)", name ? name : "<null>", id);
@@ -356,22 +431,20 @@ static int novas_calceph(const char *name, long id, double jd_tdb_high, double j
 }
 
 /**
- * Sets a ephemeris provider for Solar-system objects using the CALCEPH C library and the specified set of
- * ephemeris files. If the supplied ephemeris files contain data for major planets also, they can be used
- * by planet_calceph() / planet_calceph_hp() also, unless a separate CALCEPH ephemeris data is set via
- * novas_use_calceph_planets().
+ * Sets a ephemeris provider for Solar-system objects using the CALCEPH C library and the
+ * specified set of ephemeris files. If the supplied ephemeris files contain data for major
+ * planets also, they can be used by planet_calceph() / planet_calceph_hp() also, unless a
+ * separate CALCEPH ephemeris data is set via novas_use_calceph_planets().
  *
- * The call also make CALCEPH the default ephemeris provider for all types of Solar-system objects. If you
- * want to use another provider for major planets, you need to call set_planet_provider() /
- * set_planet_provider_hp() afterwards to specify a different provider for major planets (and Sun, Moon,
- * SSB...).
+ * The call also make CALCEPH the default ephemeris provider for all types of Solar-system
+ * objects. If you want to use another provider for major planets, you need to call
+ * set_planet_provider() / set_planet_provider_hp() afterwards to specify a different provider for
+ * the major planets (and Sun, Moon, SSB...).
  *
  * @param eph   Pointer to the CALCEPH ephemeris data that have been opened.
- * @return  0 if successful, or else -1 (errno will indicate the type of error).
+ * @return      0 if successful, or else -1 (errno will indicate the type of error).
  *
- * @sa novas_calceph_use_ids()
- * @sa novas_use_calceph_planets()
- * @sa set_ephem_provider()
+ * @sa novas_calceph_use_ids(), novas_use_calceph_planets(), set_ephem_provider()
  *
  * @author Attila Kovacs
  * @since 1.2
@@ -379,59 +452,64 @@ static int novas_calceph(const char *name, long id, double jd_tdb_high, double j
 int novas_use_calceph(t_calcephbin *eph) {
   static const char *fn = "novas_use_calceph";
 
+#if THREAD_SAFE
+  static int initialized = 0;
+
+  if(!initialized) {
+    init_lock(&bodies_mutex, 0);
+    initialized = 1;
+  }
+#endif
+
   prop_error(fn, prep_ephem(eph), 0);
 
-  // If first time, then initialize the bodies semaphore
-  if(!bodies)
-    sem_init(&sem_bodies, 0, 1);
-
   // Make sure we don't change the ephemeris provider while using it
-  prop_error(fn, mutex_lock(&sem_bodies), 0);
-
-  is_thread_safe_bodies = calceph_isthreadsafe(eph);
+  ephem_lock(&bodies_mutex);
   bodies = eph;
-  mutex_unlock(&sem_bodies);
+  ephem_unlock(&bodies_mutex);
 
   // Use CALCEPH as the default minor body ephemeris provider
   set_ephem_provider(novas_calceph);
 
   // If no planet provider is set (yet) use the same ephemeris for planets too
   // atleast until a dedicated planet provider is set.
-  if (!planets) novas_use_calceph_planets(eph);
+  if(!planets)
+    novas_use_calceph_planets(eph);
 
   return 0;
 }
 
 /**
- * Sets the CALCEPH C library and the specified ephemeris data as the ephemeris provider for the major planets
- * (and Sun, Moon, SSB...).
+ * Sets the CALCEPH C library and the specified ephemeris data as the ephemeris provider for the
+ * major planets (and Sun, Moon, SSB...).
  *
- * @param eph   Pointer to the CALCEPH ephemeris data for the major planets (including Sun, Moon, SSB...) that
- *              have been opened.
- * @return  0 if successful, or else -1 (errno will indicate the type of error).
- *
- * @sa novas_use_calceph()
- * @sa set_planet_provider()
- * @sa set_planet_provider_hp()
+ * @param eph   Pointer to the CALCEPH ephemeris data for the major planets (including Sun, Moon,
+ *              SSB...) that have been opened.
+ * @return      0 if successful, or else -1 (errno will indicate the type of error).
  *
  * @author Attila Kovacs
  * @since 1.2
+ *
+ * @sa novas_use_calceph(), set_planet_provider(), set_planet_provider_hp()
  */
 int novas_use_calceph_planets(t_calcephbin *eph) {
   static const char *fn = "novas_use_calceph_planets";
 
+#if THREAD_SAFE
+  static int initialized = 0;
+
+  if(!initialized) {
+    init_lock(&planet_mutex, 0);
+    initialized = 1;
+  }
+#endif
+
   prop_error(fn, prep_ephem(eph), 0);
 
-  // If first time, then initialize the planet semaphore
-  if(!planets)
-    sem_init(&sem_planets, 0, 1);
-
   // Make sure we don't change the ephemeris provider while using it
-  prop_error(fn, mutex_lock(&sem_planets), 0);
-
-  is_thread_safe_planets = calceph_isthreadsafe(eph);
+  ephem_lock(&planet_mutex);
   planets = eph;
-  mutex_unlock(&sem_planets);
+  ephem_unlock(&planet_mutex);
 
   // Use calceph as the default NOVAS planet provider
   set_planet_provider_hp(planet_calceph_hp);
@@ -440,3 +518,9 @@ int novas_use_calceph_planets(t_calcephbin *eph) {
   return 0;
 }
 
+
+#if __cplusplus
+#  ifdef NOVAS_NAMESPACE
+} // namespace novas
+#  endif
+#endif
