@@ -1,15 +1,12 @@
 /**
  * @file
  *
- * SuperNOVAS major planet ephemeris handler via the same generic ephemeris reader that is
- * configured by set_ephem_provider() prior to calling this routine.
+ * Major planet ephemeris lookup via the function, which was configured by set_ephem_provider().
  *
  * @date Created  on Jan 29, 2024
  * @author Attila Kovacs
  *
- * @sa solarsystem.h
- * @sa solsys-calceph.c
- * @sa solsys-cspice.c
+ * @sa ephemeris.c, solsys-calceph.c, solsys-cspice.c, orbital.c
  */
 
 #include <errno.h>
@@ -20,6 +17,13 @@
 
 #include "novas.h"
 
+#if __cplusplus
+#  ifdef NOVAS_NAMESPACE
+namespace novas {
+#  endif
+#endif
+
+
 /**
  * Major planet ephemeris data via the same generic ephemeris provider that is configured by
  * set_ephem_provider() prior to calling this routine. This is the highest precision version.
@@ -36,12 +40,11 @@
  *                       system referred to the ICRS, in AU/day.
  * @return               0 if successful, or else an error code of solarsystem_hp().
  *
- * @sa planet_ephem_provider()
- * @sa set_ephem_provider()
- * @sa solarsystem_hp()
- *
  * @since 1.0
  * @author Attila Kovacs
+ *
+ * @sa planet_ephem_provider(), set_ephem_provider()
+ * @sa solarsystem_hp()
  */
 short planet_ephem_provider_hp(const double jd_tdb[restrict 2], enum novas_planet body, enum novas_origin origin,
         double *restrict position, double *restrict velocity) {
@@ -83,7 +86,8 @@ short planet_ephem_provider_hp(const double jd_tdb[restrict 2], enum novas_plane
 /**
  * Major planet ephemeris data via the same generic ephemeris provider that is configured by
  * set_ephem_provider() prior to calling this routine. This is the regular (reduced) precision
- * version.
+ * version, but in reality it's exactly the same as the high-precision version, except for the
+ * way the TDB-based Julian date is specified.
  *
  * @param jd_tdb         [day] Barycentric Dynamical Time (TDB) based Julian date.
  * @param body           Major planet number (or that for Sun, Moon, SSB...)
@@ -93,14 +97,12 @@ short planet_ephem_provider_hp(const double jd_tdb[restrict 2], enum novas_plane
  *                       coordinates in AU referred to the ICRS.
  * @param[out] velocity  [AU/day] Velocity vector of 'body' at jd_tdb; equatorial rectangular
  *                       system referred to the ICRS, in AU/day.
- * @return               0 if successful, or else an error code of solarsystem().
- *
- * @sa planet_ephem_provider_hp()
- * @sa set_ephem_provider()
- * @sa solarsystem()
+ * @return               0 if successful, or else an error code defined by novas_planet_provider.
  *
  * @since 1.0
  * @author Attila Kovacs
+ *
+ * @sa planet_ephem_provider_hp(), set_ephem_provider()
  */
 short planet_ephem_provider(double jd_tdb, enum novas_planet body, enum novas_origin origin,
         double *restrict position, double *restrict velocity) {
@@ -109,15 +111,8 @@ short planet_ephem_provider(double jd_tdb, enum novas_planet body, enum novas_or
   return 0;
 }
 
-#if !BUILTIN_SOLSYS_EPHEM_READER
-short solarsystem(double jd_tdb, short body, short origin, double *restrict position, double *restrict velocity) {
-  prop_error("solarsystem", planet_ephem_provider(jd_tdb, body, origin, position, velocity), 0);
-  return 0;
-}
-
-short solarsystem_hp(const double jd_tdb[restrict 2], short body, short origin, double *restrict position, double *restrict velocity) {
-  prop_error("solarsystem_hp", planet_ephem_provider_hp(jd_tdb, body, origin, position, velocity), 0);
-  return 0;
-}
+#if __cplusplus
+#  ifdef NOVAS_NAMESPACE
+} // namespace novas
+#  endif
 #endif
-
