@@ -1,10 +1,14 @@
 /**
  * @file
  *
- * @author G. Kaplan and A. Kovacs
+ *  Planet calculator functions for the Earth and Sun only, with an orbital model fitted to the
+ *  DE405 ephemerides by JPL. This function is suitable for calculating the positions of Earth and
+ *  the Sun around the Solar-system Barycenter (SSB) with a typical accutacy of around 10
+ *  arcseconds within a few centuries of J2000.
  *
- *  SuperNOVAS plane calculator functions for the Earth and Sun only, with an orbital model
- *  based on the DE405 ephemerides by JPL.
+ *  It is inherently similar, at least in accuracy, to the planetary orbital models provided by @ref
+ *  planets.c, except that it can be used more widely in SuperNOVAS internally by the various
+ *  functions, which need Earth and/or Sun positions for their calculations.
  *
  *  Based on the NOVAS C Edition, Version 3.1:
  *
@@ -14,10 +18,9 @@
  *  <a href="http://www.usno.navy.mil/USNO/astronomical-applications">
  *  http://www.usno.navy.mil/USNO/astronomical-applications</a>
  *
- *  @sa solarsystem.h
- *  @sa solsys-calceph.c
- *  @sa solsys-cspice.c
- *  @sa solsys-ephem.c
+ * @author G. Kaplan and Attila Kovacs
+ *
+ * @sa ephemeris.c, planets.c, solsys-calceph.c, solsys-cspice.c, solsys-ephem.c
  */
 
 #include <math.h>
@@ -31,6 +34,14 @@
 #define T0        NOVAS_JD_J2000
 /// \endcond
 
+#if __cplusplus
+#  ifdef NOVAS_NAMESPACE
+namespace novas {
+#  endif
+#endif
+
+
+
 // Additional local function prototypes
 int sun_eph(double jd, double *restrict ra, double *restrict dec, double *restrict dis);
 
@@ -64,10 +75,9 @@ void enable_earth_sun_hp(int value) {
 }
 
 /**
- * Provides the position and velocity of the Earth and Sun only
- * at epoch 'jd_tdb' by evaluating a closed-form theory without reference to an
- * external file.  This function can also provide the position
- * and velocity of the Sun.
+ * Provides the position and velocity of the Earth and Sun only at epoch 'jd_tdb' by evaluating a
+ * closed-form theory without reference to an external file.  This function can also provide the
+ * position and velocity of the Sun.
  *
  * REFERENCES:
  * <ol>
@@ -93,11 +103,7 @@ void enable_earth_sun_hp(int value) {
  *                      1 if the input Julian date ('tjd') is out of range, 2 if 'body' is
  *                      invalid.
  *
- * @sa earth_sun_calc_hp()
- * @sa set_planet_provider()
- * @sa solarsystem()
- * @sa novas_planet_provider
- *
+ * @sa earth_sun_calc_hp(), set_planet_provider(), novas_planet_provider
  */
 short earth_sun_calc(double jd_tdb, enum novas_planet body, enum novas_origin origin,
         double *restrict position, double *restrict velocity) {
@@ -258,10 +264,10 @@ short earth_sun_calc(double jd_tdb, enum novas_planet body, enum novas_origin or
 }
 
 /**
- * It may provide the position and velocity of the Earth and Sun, the same as
- * solarsystem_earth_sun(), if enable_earth_sun_hp() is set to true (non-zero). Otherwise,
- * it will return with an error code of 3, indicating that high-precision calculations are
- * not provided by this implementation.
+ * It may provide the position and velocity of the Earth and Sun, the same as earth_sun_calc(),
+ * if enable_earth_sun_hp() is set to true (non-zero). Otherwise, it will return with an error
+ * code of 3, indicating that high-precision calculations are not provided by this
+ * implementation.
  *
  * NOTES:
  * <ol>
@@ -291,12 +297,8 @@ short earth_sun_calc(double jd_tdb, enum novas_planet body, enum novas_origin or
  *                      out of range, 2 if 'body' is invalid, or 3 if the high-precision
  *                      orbital data cannot be produced (default return value).
  *
- * @sa enable_earth_sun_hp()
- * @sa earth_sun_calc()
- * @sa set_planet_provider()
+ * @sa enable_earth_sun_hp(), earth_sun_calc(), set_planet_provider()
  * @sa solarsystem_hp()
- * @sa novas_planet_provider_hp
- *
  */
 short earth_sun_calc_hp(const double jd_tdb[restrict 2], enum novas_planet body, enum novas_origin origin,
         double *restrict position, double *restrict velocity) {
@@ -317,8 +319,8 @@ short earth_sun_calc_hp(const double jd_tdb[restrict 2], enum novas_planet body,
  * Computes equatorial spherical coordinates of Sun referred to the mean equator and equinox
  * of date.
  *
- * Quoted accuracy is 2.0 + 0.03 * T<sup>2</sup> arcsec, where T is measured in units of
- * 1000 years from J2000.0.  See reference.
+ * Quoted accuracy is 2.0 + 0.03 * T<sup>2</sup> arcsec, where T is measured in units of 1000
+ * years from J2000.0.  See reference.
  *
  * The obliquity equation is updated to equation 5.12 of the second reference.
  *
@@ -447,20 +449,9 @@ int sun_eph(double jd, double *restrict ra, double *restrict dec, double *restri
   return 0;
 }
 
-#if DEFAULT_SOLSYS == 3
-/// \cond PRIVATE
-novas_planet_provider planet_call = earth_sun_calc;
-novas_planet_provider_hp planet_call_hp = earth_sun_calc_hp;
-/// \endcond
-#elif !BUILTIN_SOLSYS3
-short solarsystem(double jd_tdb, short body, short origin, double *restrict position, double *restrict velocity) {
-  prop_error("solarsystem", earth_sun_calc(jd_tdb, body, origin, position, velocity), 0);
-  return 0;
-}
-
-short solarsystem_hp(const double jd_tdb[restrict 2], short body, short origin, double *restrict position, double *restrict velocity) {
-  prop_error("solarsystem_hp", earth_sun_calc_hp(jd_tdb, body, origin, position, velocity), 0);
-  return 0;
-}
+#if __cplusplus
+#  ifdef NOVAS_NAMESPACE
+} // namespace novas
+#  endif
 #endif
 
