@@ -24,7 +24,6 @@ namespace novas {
 #  endif
 #endif
 
-
 /// \cond PRIVATE
 #define MAX_SECONDS_DECIMALS      9                 ///< Maximum decimal places for seconds in HMS/DMS formats
 /// \endcond
@@ -267,7 +266,7 @@ void novas_tiny_rotate(const double *in, double ax, double ay, double az, double
 }
 
 /**
- * Sets the maximum number of iterations allowed for convergent inverese calculations.
+ * Sets the maximum number of iterations allowed for convergent inverse calculations.
  *
  * @param n   Maximum number of iterations allowed.
  *
@@ -278,6 +277,63 @@ void novas_set_max_iter(int n) {
   novas_inv_max_iter = n;
 }
 
+static int novas_rot(double angle, double *x, double *y) {
+  const double s = sin(angle);
+  const double c = cos(angle);
+  const double x0 = *x;
+
+  *x =  c * x0 + s * (*y);
+  *y = -s * x0 + c * (*y);
+
+  return 0;
+}
+
+/**
+ * Rotates a vector around the _x_-axis, counter clockwise when looking out from
+ * the origin along _x_.
+ *
+ * @param angle       [rad] rotation angle
+ * @param[in,out] v   Vector to rotate
+ * @return            0 if successful, or else -1 if the vector argument is NULL
+ *                    (errno set to EINVAL).
+ */
+int novas_Rx(double angle, double *v) {
+  if(!v)
+    return novas_error(-1, EINVAL, "novas_Rx", "input/output vector is NULL");
+  return novas_rot(angle, &v[1], &v[2]);
+}
+
+/**
+ * Rotates a vector around the _y_-axis, counter clockwise when looking out from
+ * the origin along _y_.
+ *
+ * @param angle       [rad] rotation angle
+ * @param[in,out] v   Vector to rotate
+ * @return            0 if successful, or else -1 if the vector argument is NULL
+ *                    (errno set to EINVAL).
+ */
+int novas_Ry(double angle, double *v) {
+  if(!v)
+    return novas_error(-1, EINVAL, "novas_Ry", "input/output vector is NULL");
+  return novas_rot(angle, &v[2], &v[0]);
+}
+
+/**
+ * Rotates a vector around the _z_-axis, counter clockwise when looking out from
+ * the origin along _z_.
+ *
+ * @param angle       [rad] rotation angle
+ * @param[in,out] v   Vector to rotate
+ * @return            0 if successful, or else -1 if the vector argument is NULL
+ *                    (errno set to EINVAL).
+ */
+int novas_Rz(double angle, double *v) {
+  if(!v)
+    return novas_error(-1, EINVAL, "novas_Rz", "input/output vector is NULL");
+  return novas_rot(angle, &v[0], &v[1]);
+}
+
+
 /// \endcond PROTECTED
 
 // ===========================================================================
@@ -534,7 +590,26 @@ double d_light(const double *pos_src, const double *pos_body) {
 }
 
 /**
- * Returns the angular separation of two locations on a sphere.
+ * Returns the angular separation of two locations on a sphere. It uses the Vincenty formula for
+ * accurate results across all locations on the sphere.
+ *
+ * NOTES:
+ * <ol>
+ * <li>Prior to version 1.5, this function used the law of cosines, which is imprecise for
+ * nearby locations</li>
+ * <li>In version 1.5, this function used the haversine formula, albeit with an error in the
+ * implementation. The haversine formula is accurate for nearby locations, but has rounding
+ * errors for antipodal locations.</li>
+ * <li>From version 1.5.1, this function uses the Vincenty formula, which is accurate for all
+ * locations on the sphere.</li>
+ * </ol>
+ *
+ * REFERENCES:
+ * <ol>
+ * <li> Vincenty, Thaddeus. "Direct and Inverse Solutions of Geodesics on the Ellipsoid with
+ * Application of Nested Equations", Survey Review. 23 (176), Directorate of Overseas Surveys,
+ * doi:10.1179/sre.1975.23.176.88.</li>
+ * </ol>
  *
  * @param lon1    [deg] longitude of first location
  * @param lat1    [deg] latitude of first location
@@ -548,10 +623,16 @@ double d_light(const double *pos_src, const double *pos_body) {
  * @sa novas_equ_sep(), novas_sun_angle(), novas_moon_angle()
  */
 double novas_sep(double lon1, double lat1, double lon2, double lat2) {
-  double c = sin(lat1 * DEGREE) * sin(lat2 * DEGREE) + cos(lat1 * DEGREE) * cos(lat2 * DEGREE) * cos((lon1 - lon2) * DEGREE);
-  double c2 = c * c;
-  // Ensure that argument to sqrt() is not negative given rounding errors.
-  return atan2(c2 < 1.0 ? sqrt(1.0 - c2) : 0.0, c) / DEGREE;
+  const double cp1 = cos(lat1 * DEGREE);
+  const double cp2 = cos(lat2 * DEGREE);
+  const double sp1 = sin(lat1 * DEGREE);
+  const double sp2 = sin(lat2 * DEGREE);
+  const double cdl = cos((lon1 - lon2) * DEGREE);
+
+  const double a = cp2 * sin((lon1 - lon2) * DEGREE);
+  const double b = cp1 * sp2 - sp1 * cp2 * cdl;
+
+  return atan2(sqrt(a*a + b * b), sp1 * sp2 + cp1 * cp2 * cdl) / DEGREE;
 }
 
 /**
@@ -750,7 +831,13 @@ int novas_print_dms(double degrees, enum novas_separator_type sep, int decimals,
       sprintf(fmt, "%%4d%%s%%02d%%s%%02d%%s");
 
     degrees = remainder(degrees, DEG360);
-    breakdown(degrees, decimals, &d, &m, &s, &ss);
+    if(degrees < 0.0) {
+      breakdown(-degrees, decimals, &d, &m, &s, &ss);
+      d = -d;
+    }
+    else {
+      breakdown(degrees, decimals, &d, &m, &s, &ss);
+    }
 
     switch(sep) {
       case NOVAS_SEP_UNITS:
